module Acts
  module AsArchivable

    def self.included(base)
      base.extend AddActsAsMethod
    end

    module AddActsAsMethod
      def acts_as_archivable(options = {})
        class_eval <<-END
           include Acts::AsArchivable::InstanceMethods
        END
      end
    end

    # Istance methods
    module InstanceMethods

      def self.included(aClass)
        aClass.extend ClassMethods
      end

      def before_update
        backup! unless self.parent
        true
      end

      # Przywraca kopię zapasową o podanym ID
      def restore!(id = nil)
        # Jesli nie podalismy ID tzn ze raczej chcemy przywrócić "siebie"
        # do oryginału - to wykonujemy na kopi zapasowej
        unless id
          orig = self.class.find(self.parent_id)

          restored = self
          # Jesli to nie jest "nasza" kopia zapasowa
          orig.backup!

          # Kopia siebie zanim przywrócimy
          clone_content(restored, orig)
          orig.parent_id = nil
          orig.save
          orig.reload
        else
          # Jesli podalismy ID to sprawdzmy czy to "nasze" dziecko
          restored = self.class.find(id)
          return unless self.id == restored.parent_id
          backup!
          # Kopia siebie zanim przywrócimy
          clone_content(restored, self)
          self.parent_id = nil
          self.save
          self.reload
        end
      end

      # Robi kopie samego siebie do archiwum
      def backup!
        backup = self.class.new
        clone_content self.class.find(self.id), backup
        backup.save
      end

      # Usuwa stare kopie pozostawiając najświeższą
      def cleanup!
        return unless self.parent_id == nil
        children_nr = self.children.size
        self.children.each_with_index { |el, index|
          el.destroy unless index >= children_nr-1
        }
        self.reload
      end

      module ClassMethods
        # Usuwa stare kopie zapasowe ze wszystkich elementów danej klasy
        def cleanup!
          # Wybierz wszystkie elementy główne
          elements = find(:all, :conditions => "parent_id IS NULL")
          elements.each { |el|
            el.cleanup!
          }
        end
      end

      private

      # Kopiuje zawartość łącznie z paperclipami
      def clone_content(source, target)
        # Nazwy charakterystyczne dla elementów paperclipa
        cont = '_content_type'
        fsize = '_file_size'
        fname = '_file_name'

        source.attributes.each { |key, value|
          next if key.to_sym == :id
          sym = "#{key}=".to_sym
          target.send(sym, value)
        }
        target.parent_id = source.id

        #target.update_attributes(source.attributes.merge(:parent_id => self.id))
        # Sprawdzmy czy ma elementy pluginu paperclip (bo je tez musimy skopiować)
        paperclips = []
        source.attributes.each { |key, val|
          # Niech zapamięta te atrybuty które mają cechy paperclipowych
          paperclips << key if key.include?(cont) || key.include?(fsize) || key.include?(fname)
        }
        paperclip_obj = []
        target.save
        # Wyciagnijmy nazwy modeli paperclipowych
        paperclips.each { |n|
          el = n.split(cont) if n.include?(cont)
          el = n.split(fname) if n.include?(fname)
          el = n.split(fsize) if n.include?(fsize)
          # Jesli istnieją elementy paperclipowe to sprawdz czy ma wszystkie
          # potrzebne tabele (bo tylko wtedy to element paperclipowy
          # i jesli tak to wrzuc to do tabeli
          if (paperclips.include?("#{el}#{cont}") &&
                paperclips.include?("#{el}#{fsize}") &&
                paperclips.include?("#{el}#{fname}"))
            paperclip_obj << el unless paperclip_obj.include?(el)
          end
        }

        # Teraz przekopiuj te atrybuty od starego starej wersji do archiwum
        paperclip_obj.each { |obj|
          sym = "#{obj.first}="
          target.send(sym, source.send(obj.first.to_s))
        }
      end
    end
  end
end

